#
from production_tcptest import *


class Gatekeeper(TestSeq):
	
	def __init__(self,host,port=2119,timeout=120):
		self.Host = host
		self.Port = port
		self.TimeOut = timeout
		TestSeq.__init__(self)
		self.Description = "Gatekeeper:%d" % self.Port
		self.addTest( TCPPortTest(host,port,60) ) 
		self.addTest( JMProbe(host,port) )
		
	def execute(self):
		## May need start,stop, try for Exception
		#Run Port Test (guaranteed to be first test)
		st_pt = self.Sequence[0].execute()
		if st_pt == ST_OK:
			st_jm = self.Sequence[1].execute()
			# Don;t really like this since ideally
			# the test sequence could return an int status
			if st_jm[0] == ST_OK:
				mgrs = self.Sequence[1].getFoundMgrs()
				if len(mgrs) > 0:
					for mgr in mgrs:
						self.addTest( JMEchoTest(self.Host,self.Port,mgr) )
					for test in self.Sequence[2:]:
						test.execute()
		return self.getStatus()


class JMProbe(TestSeq):
	
	def __init__(self,host,port):
		self.Host = host
		self.Port = port
		TestSeq.__init__(self)
		self.Description = 'Jobmanager Probe'
                mgrs  = [ 'jobmanager-condor', 'jobmanager-easymcs', \
                          'jobmanager-fork', 'jobmanager-glunix', 'jobmanager-grd', \
			  'jobmanager-loadleveler', 'jobmanager-lsf', 'jobmanager-nqe', \
                          'jobmanager-nswc', 'jobmanager-pbs', 'jobmanager-pexec',\
                          'jobmanager-prun' ]

		for mgr in mgrs:
			self.addTest( JMAuthTest(host,port,mgr))

	def getFoundMgrs(self):
		mgrs = []
		for test in self.Sequence:
			if test.getStatus() == ST_OK:
				mgrs.append(test.getMgr())
		return mgrs

class JMEchoTest(Test):
	def __init__(self,host,port,mgr):
		self.Host = host
		self.Port = port
		self.Mgr = mgr
		Test.__init__(self)
		self.Description = 'Echo Job Submission for ' + mgr
		
	def execute(self):
		self.StartTime = time.time()
		self.Status = ST_FAIL
		gram_res = '%s:%s/%s' % ( self.Host, self.Port, self.Mgr)
		self.Transcript = 'Trying Echo Job Submission for %s... ' % gram_res
		try:


                        shell = Command('globusrun -o -r %s "&(executable=/bin/echo)(arguments=Test Run)"' \
				%gram_res)
			try:
				shell.execute()
 		                if shell.getOut() == 'Test Run\n' and shell.getErr() == '':
					self.Status = ST_OK
					self.Transcript = self.Transcript + 'Succeeded.\n'
				else:
                			self.Status = ST_FAIL
					self.Transcript = self.Transcript + 'FAILED.\n'
					self.Transcript = self.Transcript + shell.getOut()
					self.Transcript = self.Transcript + shell.getErr()
			except Timeout:
				self.Status = ST_TIMEOUT
				self.Transcript = self.Transcript + "Timed out.\n"

                        self.StopTime = time.time()
                except:
                        result = ST_EXCEPTION
                        self.Transcript = self.Transcript + 'EXCEPTION Thrown\n  ' \
                                        + str(sys.exc_info()[0]) + '\n  ' \
                                        + str(sys.exc_info()[1])

                return self.Status

class JMAuthTest(Test):
        def __init__(self,host,port,mgr):
                self.Host = host
                self.Port = port
                self.Mgr = mgr
                Test.__init__(self)
                self.Description = 'Authorization Test for ' + mgr

        def getMgr(self):
                return self.Mgr

        def execute(self):
                self.StartTime = time.time()
                self.Status = ST_FAIL
                gram_res = '%s:%s/%s' % ( self.Host, self.Port, self.Mgr)
                self.Transcript = 'Trying GRAM authorization test for %s... ' % gram_res
                try:
                        shell = Command('globusrun -a -r ' + gram_res, 60)
                        try:
                                shell.execute()
                                if shell.getOut() == '\nGRAM Authentication test successful\n':
                                        self.Status = ST_OK
                                        self.Transcript = self.Transcript + 'Succeeded.\n'
                                else:
                                        self.Status = ST_FAIL
                                        self.Transcript = self.Transcript + 'FAILED.\n'
                                        self.Transcript = self.Transcript + shell.getOut()
                                        self.Transcript = self.Transcript + shell.getErr()
                        except Timeout:
                                self.Status = ST_TIMEOUT
                                self.Transcript = self.Transcript + "Timed out.\n"

                        self.StopTime = time.time()
                except:
                        result = ST_EXCEPTION
                        self.Transcript = self.Transcript + 'EXCEPTION Thrown\n  ' \
                                        + str(sys.exc_info()[0]) + '\n  ' \
                                        + str(sys.exc_info()[1])

                return self.Status













