# Base classes for a test that

import signal
import time
import sys
import os
from popen2 import *
import string

ST_OK=0
ST_NOTRUN=1
ST_TIMEOUT=2
ST_FAIL =3
ST_EXCEPTION=4


status_map = { ST_OK:'gifs/diagr1.gif', \
	       ST_NOTRUN:'gifs/diawhi.gif', \
               ST_TIMEOUT:'gifs/diaor.gif', \
	       ST_FAIL:'gifs/diared.gif', \
	       ST_EXCEPTION:'gifs/diacyan.gif' }

class Timeout:
        def __init__(self,value):
                self.value = value
        def __str__(self):
                return `self.value`

class Command:
	def __init__(self,cmd,to=60):
		self.Text = cmd
		self.TO = to
		self.Out = ''
		self.Err = ''
		self.ExitCode = -1

	def execute(self):
		signal.signal(signal.SIGALRM,AlarmHandler)
		signal.alarm(self.TO)
        	try:
                	wood = Popen3(self.Text,1)
	                self.Out = wood.fromchild.read()
        	        self.Err = wood.childerr.read()
                	self.ExitCode = wood.wait()
	        except Timeout:	
        	        os.kill(wood.pid, signal.SIGKILL)
                	self.ExitCode = wood.wait()
	                raise Timeout("Timeout occured")
        	signal.alarm(0)
		signal.signal(signal.SIGALRM,signal.SIG_DFL)
		return self.ExitCode
		
	def getOut(self):
		return self.Out
	def getErr(self):
		return self.Err
	def getExitCode(self):
		return self.ExitCode

class Test:
	def __init__(self):
		self.Status = ST_NOTRUN
		self.Description = ''
		self.StartTime = 0
		self.StopTime = 0
		self.Transcript = 'a'

	def execute(self):
		pass

	def getTranscript(self):
		return self.Transcript

	def getStatus(self):
		return self.Status
	def getDescription(self):
		return self.Description
	def getStartTime(self):
		return StartTime
	def getStopTime(self):
		return StopTime
	def getHTML(self):
		html = '<IMG ALIGN=TOP SRC="%s">' % status_map[self.Status]
		html = html + self.Description
		return html

	def getHierStatus(self):
		return (((self.Status,self.Status),self.Description),[])

class TestSeq(Test):
	def __init__(self):
		self.Status=(ST_NOTRUN,ST_NOTRUN)
		self.StartTime = 0
		self.StopTime = 0
		self.Sequence = []
		Test.__init__(self)

	def MergeStatus(self, a , b):
		from types import *
		if type(a) == TupleType and type(b) == TupleType:
			ret_val = [None, None]
			if a[0] < b[0]:
				ret_val[0] = a[0]
			else:
				ret_val[0] = b[0]
			if a[1] > b[1]:
				ret_val[1] = a[1]
			else:
				ret_val[1] = b[1]
		elif type(a) == IntType and type(b) == IntType:
			ret_val = [a,b]
			ret_val.sort()
		elif type(a) == IntType and type(b) == TupleType:
			if a < b[0]:
				ret_val = [ a, b[1]]
			elif a > b[1]:
				ret_val = [b[0],a]
			else:
				ret_val = list(b)
		else:
			if a < a[0]:
				ret_val = [ b, a[1]]
			elif b > a[1]:
				ret_val = [a[0],b]
			else:
				ret_val = list(a)
		return tuple(ret_val)
			

	def execute(self):
		self.StartTime = time.time()
		for test in self.Sequence:
			test.execute()
		self.StopTime = time.time()
		return self.getStatus()

	def addTest(self,test):
		self.Sequence.append(test)
	
	def getTranscript(self):
		text = ''
		for test in self.Sequence:
			text = text + test.getTranscript() + '\n'
		return text

	def getStatus(self):
		if len(self.Sequence) > 0:
			self.Status = self.Sequence[0].getStatus()
		for test in self.Sequence[1:]:
			self.Status = self.MergeStatus(self.Status,test.getStatus())
		return self.Status

	def getHTML(self):
		from types import *
		html = self.Description
		if type(self.Status) == IntType:
			html = Test.getHTML(self)
		else:
			html = '<IMG ALIGN=TOP SRC="%s">' % status_map[self.Status[0]]
			html = html + '<IMG ALIGN=TOP SRC="%s">' % status_map[self.Status[1]]
			html = html + self.Description
		html = html + '<ul>\n'

		for test in self.Sequence:
			html = html + test.getHTML() + '<BR>\n'
		html = html + '</ul>'
		html = string.replace(html,"<BR>\n</ul><BR>","</ul>")
		return html

	def getHierStatus(self):
		my = ( self.Status, self.Description)
		children = []
		for test in self.Sequence:
			children.append(test.getHierStatus())
		return ( my,children)
			
class TCPPortTest(Test):
	def __init__(self,host,port,Timeout=60):
		self.Host = host
		self.Port = port
		self.Timeout = Timeout
		Test.__init__(self)
		self.Description = 'TCP Port Test:%d' % self.Port
		

	def execute(self):
		try:
			import socket
			
			signal.signal(signal.SIGALRM,AlarmHandler)
			self.Status = ST_OK
			self.Transcript = 'Attempting to connect to %s:%d ...' % (self.Host,self.Port)
			self.StartTime = time.time()
	        	signal.alarm(self.Timeout)
		        try:
        		        s = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
                		s.connect(self.Host,self.Port)
		                s.close()
				self.Transcript = self.Transcript + 'SUCCESS\n'

        		except socket.error, (errnum,errtext):
                		self.Status = ST_FAIL
		                self.Transcript = self.Transcript + 'FAILED\n  ' \
                                        + errtext + '\n'
        		except Timeout:
                		self.Status = ST_TIMEOUT
	                	self.Transcript = self.Transcript + 'Timed out\n'

			self.StopTime = time.time()

		except:
			result = ST_EXCEPTION
			self.Transcript = self.Transcript + 'EXCEPTION Thrown\n  ' \
					+ str(sys.exc_info()[0]) + '\n  ' \
					+ str(sys.exc_info()[1])

		signal.alarm(0)
		signal.signal(signal.SIGALRM,signal.SIG_DFL)
        	return self.Status


def AlarmHandler(signum, Frame):
        if signum == signal.SIGALRM:
                raise Timeout("Timeout occured")
